﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace QuantumEntanglement {
    public class QuantumEntanglement {
        //All signel levels are relative  
        // 10 could be 10 uVolts, 10 milivolts, etc 
        public double DetectorNoiseSignalLevel=100;
        public bool useSquareNoiseValue=false; //square the noise amplitude -- if false, the noiselevel above should be 100
        public double DetectorThrehsold=166;
        public double SignalAmplitide=100;  // singnal amplitide before polarizrs 
        public double[] PolarizerAnglesDegrees= new double[] { 0, 22.5, 45, 67.5, 90 };

        public Random RND = new Random();  // replace this with a specific seed value if you wish 


        /// <summary>
        /// The experiment has two sides Left and rigt.
        /// Eah side has two detectors which are identified as 0 and 1
        /// 
        /// the zero detector is the cosine 
        /// 
        /// </summary>
        public int Single00=0;      // single detector on both sides fired -- Zero on both
        public int Single10=0;      // single detector on both sides fired -- 1 on the left and zero on the right
        public int Single01=0;      // single detector on both sides fired -- the zero on the left and the 1 on the right 
        public int Single11=0;      // single detector on both sides fired -- 1 on both  
        public int Neither=0;       // neither side had a detect  
        public int DoubleDetects=0; //either side had both detectors fire
        public int SingleEither=0;  // a single detector fired but only on one side 
        public int TotalRuns=0;

        /// <summary>
        /// Clears the detectin counters 
        /// </summary>
        public void Clear() {
            Single00=0;
            Single10=0;
            Single01=0;
            Single11=0;
            Neither=0;
            DoubleDetects=0;
            SingleEither=0;
            TotalRuns=0;
        }


        private double getRandomAngleInRadians() {
            return RND.NextDouble()*2*System.Math.PI;

        }


        /// <summary>
        /// The noise should always be adjusted to 100 max 
        /// 
        /// if useSquareNoiseValue then use DetectorNoiseSignalLevel=10
        /// otherwise DetectorNoiseSignalLevel=100
        /// </summary>
        /// <returns></returns>
        public double getNoiseSignal() {
            double Q=RND.NextDouble()*2*DetectorNoiseSignalLevel-DetectorNoiseSignalLevel;   // -10 to +10
            if(useSquareNoiseValue) {
                return Q*Q*Q/System.Math.Abs(Q);
            } else {
                return Q;
            }
        }

        /// <summary>
        /// returns true if signal detected in the presense of noise 
        /// </summary>
        /// <param name="signalLevel"></param>
        /// <returns></returns>
        public bool detect(double signalLevel) {
            double D =System.Math.Abs(getNoiseSignal()+signalLevel);
            return D>DetectorThrehsold;
        }


        public void Run(int numberOfRuns, double leftPolarizerAngleDeg, double rightPolarizerAngleDeg, double detectionThresh=0) {
            if(detectionThresh>20){
                DetectorThrehsold=detectionThresh; // override if 
            }
            double LR = leftPolarizerAngleDeg*System.Math.PI/180; // left polarizer angle in radians 
            double RR = rightPolarizerAngleDeg*System.Math.PI/180; // right polarizer angle in radians 
            // pre compute all the sines and cosines 
            double LC = System.Math.Cos(LR);
            double LS = System.Math.Sin(LR);
            double RC = System.Math.Cos(RR);
            double RS = System.Math.Sin(RR);

            for(int i=0; i<numberOfRuns; i++) {
                // get fauxton angle  
                double SR=getRandomAngleInRadians();
                double SC = SignalAmplitide*System.Math.Cos(SR);
                double SS = SignalAmplitide*System.Math.Sin(SR);

                // compute left side 
                // This is amount of singnal level passing through each side of polarizing splitter 
                // use dot product (cosine)
                double Left0 = LC*SC+LS*SS;
                double Left1 = LS*SC+LC*SS;

                // compute right side 
                // This is amount of singnal level passing through each side of polarizing splitter 
                // use dot product (cosine) 
                double Right0 = RC*SC+RS*SS;
                double Right1 = RS*SC+RC*SS;

                //Detect 
                bool L0 = detect(Left0);
                bool L1 = detect(Left1);
                bool R0 = detect(Right0);
                bool R1 = detect(Right1);

                TotalRuns++;
                if(L0&&L1) {
                    DoubleDetects++;
                    continue;
                }
                if(R0&&R1) {
                    DoubleDetects++;
                    continue;
                }
                if(!((L0||L1||R0||R1))) {
                    Neither++;
                    continue;
                }
                if(!((L0||L1)&&(R0||R1))) {
                    SingleEither++;
                    continue;
                }
                // if we got here it means we have a single detect on both sides 

                if(L0) {
                    if(R0) {
                        Single00++;
                    } else {
                        Single01++;
                    }
                } else {
                    if(R0) {
                        Single10++;
                    } else {
                        Single11++;
                    }
                }
            }
        }//function 


        public string report() {
            string rtn ="";
            rtn+="\r\n Single00="+Single00;
            rtn+="\r\n Single10="+Single10;
            rtn+="\r\n Single01="+Single01;
            rtn+="\r\n Single11="+Single11;
            rtn+="\r\n Neither="+Neither;
            rtn+="\r\n DoubleDetects="+DoubleDetects;
            rtn+="\r\n SingleEither="+SingleEither;
            rtn+="\r\n TotalRuns="+TotalRuns;

            rtn+="\r\n";

            int CC = CoincidentCount;
            int same = Single00+Single11;
            int diff = Single01+Single10;

            double SamePcnt = (double)same/CC*100;
            double DiffPcnt = (double)diff/CC*100;

            rtn+="\r\n Coinc Cnt="+CC;
            rtn+="\r\n Same="+SamePcnt.ToString("0.0")+" %";
            rtn+="\r\n Diff="+DiffPcnt.ToString("0.0")+" %";
            return rtn;
        }


        public int CoincidentCount {
            get {
                return Single00+Single01+Single10+Single11;  // the coincincident count 
            }
        }


        public double SamePcnt {
            get {
                int CC = CoincidentCount;
                int Q = Single00+Single11;
                return (double)Q/CC*100;
            }
        }

        public double DiffPcnt {
            get {
                int CC = CoincidentCount;
                int Q = Single01+Single10;
                return (double)Q/CC*100;
            }
        }





    }//class 

}//namespace 
